@@ -1,4 +1,4 @@
-# Copyright (C) 1998 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -22,28 +22,18 @@ mechanism (eg, pipermail) should be pointed to the right places, to do the
 archival.
 """
 
-
-#
-# system modules
-#
-import sys, os, string
+import os
+import string
+import time
 import errno
 import traceback
 
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
-
-from Mailman.Utils import reraise, mkdir, SafeDict
-
-#
-# package/project modules
-#
+from Mailman import mm_cfg
 from Mailman import Utils
 from Mailman import Mailbox
-from Mailman import mm_cfg
 from Mailman import LockFile
+from Mailman.Logging.Syslog import syslog
+from Mailman.pythonlib.StringIO import StringIO
 
 
 
@@ -53,7 +43,7 @@ def makelink(old, new):
     except os.error, e:
         code, msg = e
         if code <> errno.EEXIST:
-            reraise()
+            raise
 
 def breaklink(link):
     try:
@@ -61,7 +51,7 @@ def breaklink(link):
     except os.error, e:
         code, msg = e
         if code <> errno.ENOENT:
-            reraise()
+            raise
 
 
 
@@ -75,7 +65,7 @@ class Archiver:
 
     def InitVars(self):
 	# Configurable
-	self.archive = 1
+	self.archive = mm_cfg.DEFAULT_ARCHIVE
 	# 0=public, 1=private:
 	self.archive_private = mm_cfg.DEFAULT_ARCHIVE_PRIVATE
  	self.archive_volume_frequency = \
@@ -115,11 +105,11 @@ class Archiver:
             mm_cfg.PRIVATE_ARCHIVE_FILE_DIR,
             self._internal_name)
         try:
-            mkdir(self.private_archive_file_dir)
+            Utils.mkdir(self.private_archive_file_dir)
         except os.error, e:
             code, msg = e
             if code <> errno.EEXIST:
-                reraise()
+                raise
 
     def GetBaseArchiveURL(self):
         if self.archive_private:
@@ -164,90 +154,79 @@ class Archiver:
     #
     def __archive_to_mbox(self, post):
         """Retain a text copy of the message in an mbox file."""
-        if self.clobber_date:
-            import time
-            olddate = post.getheader('date')
-            post.SetHeader('Date', time.ctime(time.time()))
         try:
             afn = self.ArchiveFileName()
             mbox = self.__archive_file(afn)
             mbox.AppendMessage(post)
             mbox.fp.close()
         except IOError, msg:
-            self.LogMsg("error", ("Archive file access failure:\n"
-                                  "\t%s %s"
-                                  % (afn, `msg`)))
-            reraise()
-        if self.clobber_date:
-            # Resurrect original date setting.
-            post.SetHeader('Date', olddate)
+            syslog('error', 'Archive file access failure:\n\t%s %s' %
+                   (afn, msg))
+            raise
 
     def ExternalArchive(self, ar, txt):
-        d = SafeDict({'listname': self.real_name})
+        d = Utils.SafeDict({'listname': self.internal_name()})
         cmd = ar % d
         extarch = os.popen(cmd, 'w')
         extarch.write(txt)
-        extarch.close()
+        status = extarch.close()
+        if status:
+            syslog('error', 'external archiver non-zero exit status: %d\n' %
+                   (status & 0xff00) >> 8)
 
     #
     # archiving in real time  this is called from list.post(msg)
     #
-    def ArchiveMail(self, msg):
+    def ArchiveMail(self, msg, msgdata):
         """Store postings in mbox and/or pipermail archive, depending."""
 	# Fork so archival errors won't disrupt normal list delivery
         if mm_cfg.ARCHIVE_TO_MBOX == -1:
             return
-	if os.fork(): 
-	    return
-        # archive to builtin html archiver.  first grab the archiver lock
-        lockfile = os.path.join(mm_cfg.LOCK_DIR, self._internal_name) + \
-                   '.archiver.lock'
-        lock = LockFile.LockFile(lockfile, lifetime=60*5)
-        lock.lock()
+        #
+        # We don't need an extra archiver lock here because we know the list
+        # itself must be locked.
+        if mm_cfg.ARCHIVE_TO_MBOX in (1, 2):
+            self.__archive_to_mbox(msg)
+            if mm_cfg.ARCHIVE_TO_MBOX == 1:
+                # Archive to mbox only.
+                return
+        # From this point on, we're doing all the expensive archiving work.
+        # If anything goes wrong here, we will simply log this and let the
+        # normal delivery mechanism continue.  The archiver is too f*cked up
+        # anyway, and at the very least we've got the mbox to regenerate
+        # from.
+        t0 = time.time()
         try:
-            try:
-                if mm_cfg.ARCHIVE_TO_MBOX in [1, 2]:
-                    self.__archive_to_mbox(msg)
-                    if mm_cfg.ARCHIVE_TO_MBOX == 1:
-                        # Archive to mbox only.
-                        os._exit(0)
-                # from this point on, we're doing all the expensive archiving
-                # work.
-                txt = msg.unixfrom
-                for h in msg.headers:
-                    txt = txt + h
-                if msg.body[0] != '\n':
-                    txt = txt + "\n"
-                for line in string.split(msg.body, '\n'):
-                    # Quote unprotected From_ lines appearing in body
-                    if line and line[:5] == 'From ':
-                        line = '>' + line
-                    txt = txt + "%s\n" % line
-                # should we use the internal or external archiver?
-                private_p = self.archive_private
-                if mm_cfg.PUBLIC_EXTERNAL_ARCHIVER and not private_p:
-		    self.ExternalArchive(mm_cfg.PUBLIC_EXTERNAL_ARCHIVER, txt)
-                elif mm_cfg.PRIVATE_EXTERNAL_ARCHIVER and private_p:
-                    self.ExternalArchive(mm_cfg.PRIVATE_EXTERNAL_ARCHIVER, txt)
-                else:
-                    # use the internal archiver
-                    f = StringIO(txt)
-                    import HyperArch
-                    h = HyperArch.HyperArchive(self)
-                    h.processUnixMailbox(f, HyperArch.Article)
-                    h.close()
-                    f.close()
-            except:
-                traceback.print_exc(file=sys.stderr)
-        finally:
-            # it's still possible to take a long time to run the archiver :(
-            try:
-                lock.unlock()
-            except LockFile.NotLockedError:
-                pass
-            # need this or we'll never see the error messages!
-            sys.stderr.flush()
-            os._exit(0)
+            txt = msg.unixfrom
+            for h in msg.headers:
+                txt = txt + h
+            if not msg.body or msg.body[0] <> '\n':
+                txt = txt + "\n"
+            for line in string.split(msg.body, '\n'):
+                # Quote unprotected From_ lines appearing in body
+                if line and line[:5] == 'From ':
+                    line = '>' + line
+                txt = txt + "%s\n" % line
+            # should we use the internal or external archiver?
+            private_p = self.archive_private
+            if mm_cfg.PUBLIC_EXTERNAL_ARCHIVER and not private_p:
+                self.ExternalArchive(mm_cfg.PUBLIC_EXTERNAL_ARCHIVER, txt)
+            elif mm_cfg.PRIVATE_EXTERNAL_ARCHIVER and private_p:
+                self.ExternalArchive(mm_cfg.PRIVATE_EXTERNAL_ARCHIVER, txt)
+            else:
+                # use the internal archiver
+                f = StringIO(txt)
+                import HyperArch
+                h = HyperArch.HyperArchive(self)
+                h.processUnixMailbox(f, HyperArch.Article)
+                h.close()
+                f.close()
+        except:
+            traceback.print_exc()
+            syslog('error', 'CORRUPT ARCHIVE FOR LIST: %s' %
+                   self.internal_name())
+        else:
+            t1 = time.time()
 	
     #
     # called from MailList.MailList.Save()
