@@ -1,4 +1,4 @@
-# Copyright (C) 1998 by the Free Software Foundation, Inc.
+# Copyright (C) 1998,1999,2000 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -19,36 +19,21 @@
 
 
 import os
-import string
 import time
-import types
-import Crypt
-import Errors
-import Utils
-import Cookie
+import sha
+import marshal
+from types import StringType, TupleType
 from urlparse import urlparse
-import mm_cfg
 
-# TBD: is this the best location for the site password?
-SITE_PW_FILE = os.path.join(mm_cfg.DATA_DIR, 'adm.pw')
+from Mailman import Crypt
+from Mailman import Errors
+from Mailman import Utils
+from Mailman import Cookie
+from Mailman import mm_cfg
 
 
+
 class SecurityManager:
-    def SetSiteAdminPassword(self, pw):
-        fp = Utils.open_ex(SITE_PW_FILE, 'w', perms=0640)
-        fp.write(Crypt.crypt(pw, Utils.GetRandomSeed()))
-        fp.close()
-
-    def CheckSiteAdminPassword(self, str):
-	try:
-	    f = open(SITE_PW_FILE, "r")
-	    pw = f.read()
-	    f.close()
-	    return Crypt.crypt(str, pw) == pw
-	# There probably is no site admin password if there was an exception
-	except: 
-	    return 0
-
     def InitVars(self, crypted_password):
 	# Configurable, however, we don't pass this back in GetConfigInfo
 	# because it's a special case as it requires confirmation to change.
@@ -57,9 +42,9 @@ class SecurityManager:
 	self.passwords = {}
 
     def ValidAdminPassword(self, pw):
-	if self.CheckSiteAdminPassword(pw):
+	if Utils.CheckSiteAdminPassword(pw):
             return 1
-	return type(pw) == types.StringType and \
+	return type(pw) == StringType and \
                Crypt.crypt(pw, self.password) == self.password
 
     def ConfirmAdminPassword(self, pw):
@@ -68,71 +53,81 @@ class SecurityManager:
 	return 1
 
     def WebAuthenticate(self, user=None, password=None, cookie=None):
+        key = self.internal_name()
         if cookie:
-            cookie_key = self._internal_name + ':' + cookie
-        else:
-            cookie_key = self._internal_name
-        if password is not None:  # explicit login
+            key = key + ':' + cookie
+        # password will be None for explicit login
+        if password is not None:
             if user:
                 self.ConfirmUserPassword(user, password)
             else:
                 self.ConfirmAdminPassword(password)
-            print self.MakeCookie(cookie_key)
+            print self.MakeCookie(key)
             return 1
         else:
-            return self.CheckCookie(cookie_key)
+            return self.CheckCookie(key)
 
     def MakeCookie(self, key):
-        # Make sure we have the necessary ingredients for our cookie
-        client_ip = os.environ.get('REMOTE_ADDR') or '0.0.0.0'
-        issued = int(time.time())
-        expires = issued + mm_cfg.ADMIN_COOKIE_LIFE
-        # ... including the secret ingredient :)
+        # Ingredients for our cookie: our `secret' which is the list's admin
+        # password (never sent in the clear) and the time right now in seconds
+        # since the epoch.
         secret = self.password
-        mac = hash(secret + client_ip + `issued` + `expires`)
-        # Mix all ingredients gently together,
+        issued = int(time.time())
+        # Get a digest of the secret, plus other information.
+        mac = sha.new(secret + `issued`).hexdigest()
+        # Create the cookie object.  The way the cookie module converts
+        # non-strings to pickles can cause problems if the resulting string
+        # needs to be quoted.  So we'll do the conversion ourselves.
         c = Cookie.Cookie()
-        c[key] = [client_ip, issued, expires, mac]
-        # place in oven,
-        path = urlparse(self.web_page_url)[2] # '/mailman'
+        c[key] = Utils.hexlify(marshal.dumps((issued, mac)))
+        # The path to all Mailman stuff, minus the scheme and host,
+        # i.e. usually the string `/mailman'
+        path = urlparse(self.web_page_url)[2]
         c[key]['path'] = path
-        # and bake until golden brown
-        c[key]['expires'] = mm_cfg.ADMIN_COOKIE_LIFE
+        # Should we use session or persistent cookies?
+        if mm_cfg.ADMIN_COOKIE_LIFE > 0:
+            c[key]['expires'] = mm_cfg.ADMIN_COOKIE_LIFE
+            c[key]['max-age'] = mm_cfg.ADMIN_COOKIE_LIFE
+        # Set the RFC 2109 required header
+        c[key]['version'] = 1
+        return c
+
+    def ZapCookie(self, cookie=None):
+        key = self.internal_name()
+        if cookie:
+            key = key + ':' + cookie
+        # Logout of the session by zapping the cookie.  For safety both set
+        # max-age=0 (as per RFC2109) and set the cookie data to the empty
+        # string.
+        c = Cookie.Cookie()
+        c[key] = ''
+        # The path to all Mailman stuff, minus the scheme and host,
+        # i.e. usually the string `/mailman'
+        path = urlparse(self.web_page_url)[2]
+        c[key]['path'] = path
+        c[key]['max-age'] = 0
+        # Don't set expires=0 here otherwise it'll force a persistent cookie
+        c[key]['version'] = 1
         return c
 
     def CheckCookie(self, key):
         cookiedata = os.environ.get('HTTP_COOKIE')
         if not cookiedata:
             return 0
-        #
-        # TBD: At least some versions of MS Internet Explorer stores cookies
-        # without the double quotes.  This has been verified for MSIE 4.01,
-        # although MSIE 5 is fixed.  The bug (see PR#80) is that if these
-        # double quotes are missing, the cookie data does not unpickle into
-        # the list that we expect.  The kludge given here (slightly modified)
-        # was initially provided by Evaldas Auryla <evaldas.auryla@pheur.org>
-        #
-##        self.LogMsg('debug', 'Browser Cookie: ' + cookiedata)
-        keylen = len(key)
-        if cookiedata[keylen+1] <> '"' and cookiedata[-1] <> '"':
-            cookiedata = key + '="' + cookiedata[keylen+1:] + '"'
-##            self.LogMsg('debug', 'Using   Cookie: ' + cookiedata)
         c = Cookie.Cookie(cookiedata)
         if not c.has_key(key):
             return 0
-        cookie = c[key].value
-        if (type(cookie) <> type([]) or
-            len(cookie) <> 4):
+        # Undo the encoding we performed in MakeCookie() above
+        try:
+            cookie = marshal.loads(Utils.unhexlify(c[key].value))
+            issued, received_mac = cookie
+        except (EOFError, ValueError, TypeError):
             raise Errors.MMInvalidCookieError
-        client_ip = os.environ.get('REMOTE_ADDR') or '0.0.0.0'
         now = time.time()
-        [for_ip, issued, expires, received_mac] = cookie
-        if (for_ip <> client_ip or now < issued):
+        if now < issued:
             raise Errors.MMInvalidCookieError
-        if now > expires:
-            raise Errors.MMExpiredCookieError
         secret = self.password
-        mac = hash(secret + client_ip + `issued` + `expires`)
+        mac = sha.new(secret + `issued`).hexdigest()
         if mac <> received_mac:
             raise Errors.MMInvalidCookieError
         return 1
